package edu.berkeley.cs160.group7.BeenThereDoneThatHiFi;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ScrollView;
import android.widget.Toast;

public class Launcher extends Activity {
	
	// Shared preferences file where all the game data like username will be stored
	public static final String STORAGE_FILE = "btdt_data";
	public static SharedPreferences mGameData = null;
    public static SharedPreferences.Editor mGameDataEditor = null;

	// The two activities that can be called from Launcher
	private static final int ACTIVITY_MAINMENU = 0;
	private static final int ACTIVITY_NEWACCOUNT = 1;
	
	// The two views in Launcher (intro scene and the about page)
	private static final int LAUNCHER_MAIN_VIEW = 0;
	private static final int LAUNCHER_ABOUT_VIEW = 1;
	
	// The buttons in LAUNCHER_MAIN_VIEW
    private Button mStartButton, mAboutButton;
    
    // The scroller in LAUNCHER_ABOUT_VIEW
    private ScrollView mAboutScroll;
    
    // misc info regarding the launcher activity...mostly for lifecycle handling
    private int mCurrentView = LAUNCHER_MAIN_VIEW;
    private int mCurrentYCoord = 0;
    private View mCurrentFocus = null;
    
    // Called when the activity is first created.
    @Override 
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        mGameData = getSharedPreferences(STORAGE_FILE,MODE_PRIVATE);
        mGameDataEditor = mGameData.edit();
        
        if(icicle != null){
        	mCurrentView = icicle.getInt("current_view");
        	changeView(mCurrentView);
        	if(icicle.keySet().contains("focus_id")){
        		mCurrentFocus = findViewById(icicle.getInt("focus_id"));
        		mCurrentFocus.requestFocus();
        	}
			if(mCurrentView == LAUNCHER_ABOUT_VIEW){
        		mCurrentYCoord = icicle.getInt("y_coord");
        		mAboutScroll.scrollTo(0, mCurrentYCoord);
        	}
        }else{
        	changeView(LAUNCHER_MAIN_VIEW);
        }
    }
    
    @Override
	protected void onResume() {
		super.onResume();
		if(mCurrentFocus != null){
			mCurrentFocus.requestFocus();
		}
		if(mCurrentView == LAUNCHER_ABOUT_VIEW){
			mAboutScroll.scrollTo(0, mCurrentYCoord);
		}
	}

	@Override
	protected void onFreeze(Bundle outState) {
		super.onFreeze(outState);
		if(mCurrentView == LAUNCHER_ABOUT_VIEW){
			mCurrentYCoord = mAboutScroll.getScrollY();
			outState.putInt("y_coord", mCurrentYCoord);
		}
		outState.putInt("current_view", mCurrentView);	
		mCurrentFocus = getCurrentFocus();
		if(mCurrentFocus != null){
			outState.putInt("focus_id", mCurrentFocus.getId());
		}
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		mCurrentFocus = getCurrentFocus();
		if(mCurrentView == LAUNCHER_ABOUT_VIEW){
			mCurrentYCoord = mAboutScroll.getScrollY();
		}
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode,
			String data, Bundle extras) {
		switch (requestCode) {
		case ACTIVITY_NEWACCOUNT:
			// update selected item on closing of the subactivity
			if (resultCode == RESULT_OK){
				
				mGameDataEditor.putString("onlineId", extras.getString("onlineId"));
				mGameDataEditor.putString("Username", extras.getString("Username"));
				mGameDataEditor.putInt("Avatar", extras.getInt("Avatar"));
				mGameDataEditor.putString("ItemsCount", extras.getString("ItemsCount"));
				mGameDataEditor.putString("PointsCount", extras.getString("PointsCount"));
				mGameDataEditor.putString("Level", extras.getString("Level"));
				mGameDataEditor.commit();
				start_game();
			} return;
		case ACTIVITY_MAINMENU:
			// update search results (or boot back to search activity if unable
			// to find the group/user
			if (resultCode == RESULT_FIRST_USER) {
				reset_game();
			} return;
		default:
			super.onActivityResult(requestCode, resultCode, data, extras);
		}
	}
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK:
			if(mCurrentView == LAUNCHER_ABOUT_VIEW){
				mCurrentYCoord = mAboutScroll.getScrollY();
				changeView(LAUNCHER_MAIN_VIEW);
			}else{
				finish();
			}
			return true;
		default:
			return super.onKeyDown(keyCode, event);
		}
	}
	
	private void changeView(int viewid){
		if(viewid == LAUNCHER_MAIN_VIEW){
			setContentView(R.layout.launcher);
			mCurrentView = LAUNCHER_MAIN_VIEW;
			mStartButton = (Button) findViewById(R.id.start_button);
			mAboutButton = (Button) findViewById(R.id.about_button);
			
	        mStartButton.setOnClickListener(
	                new OnClickListener() {
	                    public void onClick(View v) {
	                        start_game();
	                    }
	                });
	        mAboutButton.setOnClickListener(
	                new OnClickListener() {
	                    public void onClick(View v) {
	                        changeView(LAUNCHER_ABOUT_VIEW);
	                    }
	                });
	        mStartButton.requestFocus();
	        mCurrentFocus = getCurrentFocus();
		}
		if(viewid == LAUNCHER_ABOUT_VIEW){
			setContentView(R.layout.launcher_about);
			mCurrentView = LAUNCHER_ABOUT_VIEW;
			mAboutScroll = (ScrollView) findViewById(R.id.about_scroll);
			mAboutScroll.requestFocus();
			mCurrentFocus = getCurrentFocus();
			mAboutScroll.scrollTo(0, mCurrentYCoord);
		}
	}
	
	private void start_game(){
    	Intent i;
    	if(mGameData.getString("Username",null) == null ||   
    			mGameData.getInt("Avatar", -1) == -1){
    		i = new Intent(Launcher.this,NewAccount.class);
    		startSubActivity(i, ACTIVITY_NEWACCOUNT);
    	}else{
    		i = new Intent(Launcher.this,MainMenu.class);
    		startSubActivity(i, ACTIVITY_MAINMENU);
    	}
    }
	
    private void reset_game(){	
		mCurrentView = LAUNCHER_MAIN_VIEW;
	    mCurrentYCoord = 0;
	    mGameDataEditor.clear();
	    mGameDataEditor.commit();
        stopService(new Intent(Launcher.this, 
				ItemScanner.class));
		Toast.makeText(Launcher.this,"Game has been reset.",Toast.LENGTH_SHORT).show();
    }
}