package edu.berkeley.cs160.group7.BeenThereDoneThatHiFi;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.StringTokenizer;

import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.os.Bundle;
import android.view.KeyEvent;
import android.widget.Toast;

import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayController;
import com.google.android.maps.Point;


public class MapBrowser extends MapActivity {
	
	final public static int DEFAULT_LATITUDE = (int) 37.875e6;
	final public static int DEFAULT_LONGITUDE = (int) -122.26e6;
	final private static int GAME_ZOOMLEVEL = 16;
	
	public static final String PREF_LATITUDE = "CurrentLatitude";
	public static final String PREF_LONGITUDE = "CurrentLongitude";
	
	// Shared preferences file where all the game data like username will be stored
	public static SharedPreferences mGameData = null;
    public static SharedPreferences.Editor mGameDataEditor = null;
    
	private static MapView mMap;
	private static MapController mMapControl;
	private static PersonOverlay mLocationOverlay;
	private static ItemsOverlay mItemLocationsOverlay;
	private static ArrowsOverlay mArrowDirectionsOverlay;
	private static OverlayController mOverlayControl;
	
	private Point mCurrentLocation;
	
	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.mapbowser_map);
		mGameData = getSharedPreferences(Launcher.STORAGE_FILE,MODE_PRIVATE);
        mGameDataEditor = mGameData.edit();
        
		if(mGameData.getBoolean("FirstRun", true)){
			mCurrentLocation = new Point(DEFAULT_LATITUDE,DEFAULT_LONGITUDE);
			mGameDataEditor.putBoolean("FirstRun", false);
			mGameDataEditor.putInt(PREF_LATITUDE, mCurrentLocation.getLatitudeE6());
			mGameDataEditor.putInt(PREF_LONGITUDE, mCurrentLocation.getLongitudeE6());
			mGameDataEditor.commit();
		}else{
			mCurrentLocation = new Point(mGameData.getInt(PREF_LATITUDE, DEFAULT_LATITUDE),
										mGameData.getInt(PREF_LONGITUDE, DEFAULT_LONGITUDE));			
		}
		
		mMap = (MapView) findViewById(R.id.location_map);
		mMapControl = mMap.getController();
		mMapControl.centerMapTo(mCurrentLocation, true);
		mMapControl.zoomTo(GAME_ZOOMLEVEL);
		mMap.preLoad();
		ArrayList<Item> visibleItems = getVisibleItems(mCurrentLocation); // items that will have an overlay drawn are visible
		ArrayList<Item> ofScreenItems = getOfScreenItems(mCurrentLocation); // items of screen that will get arrows
		generateOverlays(visibleItems,ofScreenItems);
		
		mOverlayControl = mMap.createOverlayController();
		mOverlayControl.add(mLocationOverlay,false);	
		mOverlayControl.add(mItemLocationsOverlay, false);
		mOverlayControl.add(mArrowDirectionsOverlay, false);
		activateOverlays();
		
	}
	
	@Override
	protected void onFreeze(Bundle outState) {
		super.onFreeze(outState);
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		mGameDataEditor.putInt(PREF_LATITUDE, mCurrentLocation.getLatitudeE6());
		mGameDataEditor.putInt(PREF_LONGITUDE, mCurrentLocation.getLongitudeE6());
		mGameDataEditor.commit();
	}


	@Override
	protected void onResume() {
		super.onResume();
	}
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch(keyCode){
		case KeyEvent.KEYCODE_DPAD_UP:
			changeCurrentLocation(1000,0);
			break;
		case KeyEvent.KEYCODE_DPAD_DOWN:
			changeCurrentLocation(-1000,0);
			break;
		case KeyEvent.KEYCODE_DPAD_LEFT:
			changeCurrentLocation(0,-1000);
			break;
		case KeyEvent.KEYCODE_DPAD_RIGHT:
			changeCurrentLocation(0,1000);
			break;
		case KeyEvent.KEYCODE_DPAD_CENTER:
			attemptItemPickUp();
			break;
		}
		return super.onKeyDown(keyCode, event);
	}
	private void attemptItemPickUp(){
		ArrayList<Item> visibleItems = getVisibleItems(mCurrentLocation);
		int [] myXY = new int[2];
		getPointXY(mMap,mCurrentLocation,myXY);
		ArrayList<Item> toPickUp = null;
		for(Item i: visibleItems){
			int[] xyCoords = new int[2];	
			getPointXY(mMap,i.mCoords,xyCoords);
			double distance = calculateDistance(myXY,xyCoords);
			//Toast.makeText(this, distance+","+i.mPickUpRadius, Toast.LENGTH_LONG).show();
			if(distance <= i.mPickUpRadius+20){ //+20 is for the little circle around the man
				if(toPickUp == null){
					toPickUp = new ArrayList<Item>();
				}
				toPickUp.add(i);
			}
		}
		if(toPickUp != null){
			for(Item p : toPickUp){
				final Item tmp = p;
				DialogInterface.OnClickListener lis = new DialogInterface.OnClickListener(){
					public void onClick(DialogInterface dialog, int which){
						if(which == -1){
							
							
							pickUpItem(tmp);
							boolean fusedItem = checkFusedItem(tmp );
							if(fusedItem){
								ArrayList<Item> requiredItems = getRequiredItem(tmp);
								ArrayList<Item> superItem = getSuperItem(tmp);
								String showRequiredItems = "";
								String showSupterItem = "";
								for(int i=0; i<requiredItems.size(); i++){
									if(i!=0){
										showRequiredItems+=", ";
									}
									showRequiredItems = showRequiredItems + requiredItems.get(i).mName;
									showSupterItem = superItem.get(0).mName;
								}
								
								showAlert("Item \""+tmp.mName+"\"",0,"is being fused with \""+showRequiredItems+"\" to create \""+showSupterItem+"\".",
										  "",this,"OK",null,true,null);
							}
							Toast.makeText(MapBrowser.this,"Item \""+tmp.mName+"\" has been picked up.",Toast.LENGTH_LONG).show();
							changeCurrentLocation(0,0); // hack to update overlays
						}else if(which == -2){
							Toast.makeText(MapBrowser.this,"Item \""+tmp.mName+"\" will be ignored for now.",Toast.LENGTH_LONG).show();
						}else{

							showAlert("Item \""+tmp.mName+"\" Description.",0,tmp.mDescription+"\n\nWhat do you want to do with the item?",
									  "Pick Up",this,"Ignore",this,false,null);
						}
					}
				};	
				showAlert("Item \""+p.mName+"\" Found.",0,"What do you want to do with the item?",
						  "Pick Up", lis, "Ignore", lis, "Examine", lis,false, null);
			}
		}else{
			Toast.makeText(this, "No item is close enough to pickup or examine.", Toast.LENGTH_LONG).show();
		}
	}
	
	private double calculateDistance(int[] xy1, int[] xy2){
		return Math.sqrt(Math.pow(xy1[0]-xy2[0],2)+Math.pow(xy1[1]-xy2[1], 2));
	}
	
	private void getPointXY(MapView map, Point p, int[] xy){
		Point mapCenter = map.getMapCenter();
		
		int mapWidth = map.getWidth();
		int longSpan = map.getLongitudeSpan();
		xy[0] = (int) (mapWidth*(p.getLongitudeE6()-mapCenter.getLongitudeE6()+longSpan/2.0)/longSpan);
		
		int mapHeight = map.getHeight();	
		int latSpan = map.getLatitudeSpan();
		xy[1] = (int) (mapHeight*(-p.getLatitudeE6()+mapCenter.getLatitudeE6()+latSpan/2.0)/latSpan);
	}

	private void changeCurrentLocation(int deltaLat, int deltaLong){
		mCurrentLocation = new Point(mCurrentLocation.getLatitudeE6()+deltaLat,
										mCurrentLocation.getLongitudeE6()+deltaLong);
		mMapControl.centerMapTo(mCurrentLocation, false);
		mMapControl.zoomTo(GAME_ZOOMLEVEL);
		mMap.preLoad();
		mOverlayControl.clear();
		ArrayList<Item> visibleItems = getVisibleItems(mCurrentLocation); // items that will have an overlay drawn are visible
		ArrayList<Item> ofScreenItems = getOfScreenItems(mCurrentLocation); // items of screen that will get arrows
		generateOverlays(visibleItems,ofScreenItems);
		
		mOverlayControl = mMap.createOverlayController();
		mOverlayControl.add(mLocationOverlay, false);	
		mOverlayControl.add(mItemLocationsOverlay, false);
		mOverlayControl.add(mArrowDirectionsOverlay, false);
	}
	
	// the following 4 functions as server versions...the prototype headers can change...just make
	// sure to update the rest of the code accordingly
	private ArrayList<Item> getVisibleItems(Point p){
		// server side code to get items in square around the person should go here
		// you can kill the reset of stuff I wrote...thats just for testing
		ArrayList<Item> items = new ArrayList<Item>();
		items = getItems(p, 0, mGameData);
		return items;
	}
	private ArrayList<Item> getOfScreenItems(Point p){
		// server side code to get items in large circle around the person should go here
		// this list will only have items that can be seen (not the items that are in view)
		ArrayList<Item> items = new ArrayList<Item>();
		items = getItems(p, 1, mGameData);
		return items;
	}
	
	// Option 0 = Nearby Items Only
	// Option 1 = Far Items Only
	// Option 2 = Far & Nearby Items
	public static ArrayList<Item> getItems(Point p, int option, SharedPreferences gameData){
		ArrayList<Item> items = new ArrayList<Item>();
		String line="";
        String temp="";
        String id = "0";
        String lon = "0";
        String lat = "0";
        String name = "0";
        String description = "0";
        String pick_distance = "0";
        String point_value="";
        String iconURL = "";
        String superId = "";
        String requiredItemsList ="";
        double midToLeft1 = 0;
        double midToRight1 = 0;
        double midToTop1 = 0;
        double midToBottom1 = 0;
        double midToLeft2 = 0;
        double midToRight2 = 0;
        double midToTop2 = 0;
        double midToBottom2 = 0;
        double yourLon = p.getLongitudeE6();
        double yourLat = p.getLatitudeE6();
        double doubleLon =   yourLon /1000000;
        double doubleLat =   yourLat /1000000;
        String onlineId = gameData.getString("onlineId",null);


		midToLeft1 = doubleLon - 0.0065;
		midToRight1 = doubleLon + 0.0065;
		midToTop1 = doubleLat + 0.004;
		midToBottom1 = doubleLat - 0.004;

		midToLeft2 = doubleLon - 0.026;
		midToRight2 = doubleLon + 0.026;
		midToTop2 = doubleLat + 0.016;
		midToBottom2 = doubleLat - 0.016;
	
		try{
        	temp = "";
        	line = "";
        	String query = "http://khoaphung.com/android/getItems.jsp?midToLeft1="+midToLeft1+"&midToRight1="+midToRight1+"&midToTop1="+midToTop1+"&midToBottom1="+midToBottom1+"&midToLeft2="+midToLeft2+"&midToRight2="+midToRight2+"&midToTop2="+midToTop2+"&midToBottom2="+midToBottom2+"&onlineId="+onlineId+"&option="+option+"";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL (query).openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
            StringTokenizer st = new StringTokenizer(temp, ";");
            while(st.hasMoreTokens()){
                id = st.nextToken();
                lat = st.nextToken();
                lon = st.nextToken();
                name = st.nextToken();
                description = st.nextToken();
                pick_distance = st.nextToken();
                point_value = st.nextToken();
                iconURL = st.nextToken();
                superId = st.nextToken();
                if(superId.equals("none")){
                	superId = "0";
                }
                requiredItemsList = st.nextToken();
                ArrayList<Long> RequiredItemIds = new ArrayList<Long>();
                if(!requiredItemsList.equals("none")){
	                StringTokenizer st2 = new StringTokenizer(requiredItemsList, ",");
	                while(st2.hasMoreTokens()){
	                	String temp2 = st2.nextToken();
	                	RequiredItemIds.add(Long.parseLong(temp2.trim()));
	                }
                }
                items.add(new Item( Long.parseLong(id.trim()),
                                   new Point((int) (Double.valueOf(lat.trim()) * 1000000),(int) (Double.valueOf(lon.trim()) * 1000000)),
                                   name,
                                   description,
                                   (int) (Double.valueOf(pick_distance.trim()).longValue()),
                                   Integer.parseInt(point_value.trim()),
                                   iconURL,
                                   Long.parseLong(superId.trim()),
                                   RequiredItemIds
               					  )
               			);      
            }
        }catch(Exception e){
        }
		return items;
	}
	
	public static ArrayList<Item> getUserItem(SharedPreferences gameData){
		// server side code to get items in large circle around the person should go here
		// this list will only have items that can be seen (not the items that are in view)
		ArrayList<Item> items = new ArrayList<Item>();
		String username = gameData.getString("Username",null);
		String onlineId = gameData.getString("onlineId",null);
		String temp;
		String line;
		String id = "0";
        String lon = "0";
        String lat = "0";
        String name = "0";
        String description = "0";
        String pick_distance = "0";
        String point_value="";
        String iconURL = "";
        String superId = "";
        String requiredItemsList ="";
		
		try{
        	temp = "";
        	line = "";
        	String query = "http://khoaphung.com/android/getUserItem2.jsp?onlineId="+onlineId+"&username="+username+"";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL (query).openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
            StringTokenizer st = new StringTokenizer(temp, ";");
            while(st.hasMoreTokens()){
                id = st.nextToken();
                lat = st.nextToken();
                lon = st.nextToken();
                name = st.nextToken();
                description = st.nextToken();
                pick_distance = st.nextToken();
                point_value = st.nextToken();
                iconURL = st.nextToken();
                superId = st.nextToken();
                if(superId.equals("none")){
                	superId = "0";
                }
                requiredItemsList = st.nextToken();
                ArrayList<Long> RequiredItemIds = new ArrayList<Long>();
                if(!requiredItemsList.equals("none")){
	                StringTokenizer st2 = new StringTokenizer(requiredItemsList, ",");
	                while(st2.hasMoreTokens()){
	                	String temp2 = st2.nextToken();
	                	RequiredItemIds.add(Long.parseLong(temp2.trim()));
	                }
                }
                items.add(new Item( Long.parseLong(id.trim()),
                                   new Point((int) (Double.valueOf(lat.trim()) * 1000000),(int) (Double.valueOf(lon.trim()) * 1000000)),
                                   name,
                                   description,
                                   (int) (Double.valueOf(pick_distance.trim()).longValue()),
                                   Integer.parseInt(point_value.trim()),
                                   iconURL,
                                   Long.parseLong(superId.trim()),
                                   RequiredItemIds
               					  )
               			);      
            }
        }catch(Exception e){
        }
		return items;
	}	
	
	public static ArrayList<Item> getRequiredItem(Item itm){
		// server side code to get items in large circle around the person should go here
		// this list will only have items that can be seen (not the items that are in view)
		ArrayList<Item> items = new ArrayList<Item>();
		String temp;
		String line;
		String id = "0";
        String lon = "0";
        String lat = "0";
        String name = "0";
        String description = "0";
        String pick_distance = "0";
        String point_value="";
        String iconURL = "";
        String superId = "";
        String requiredItemsList ="";
        long itemId = itm.mId;
		
		try{
        	temp = "";
        	line = "";
        	String query = "http://khoaphung.com/android/getRequiredItems.jsp?itemId="+itemId+"";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL (query).openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
            StringTokenizer st = new StringTokenizer(temp, ";");
            while(st.hasMoreTokens()){
                id = st.nextToken();
                lat = st.nextToken();
                lon = st.nextToken();
                name = st.nextToken();
                description = st.nextToken();
                pick_distance = st.nextToken();
                point_value = st.nextToken();
                iconURL = st.nextToken();
                superId = st.nextToken();
                if(superId.equals("none")){
                	superId = "0";
                }
                requiredItemsList = st.nextToken();
                ArrayList<Long> RequiredItemIds = new ArrayList<Long>();
                if(!requiredItemsList.equals("none")){
	                StringTokenizer st2 = new StringTokenizer(requiredItemsList, ",");
	                while(st2.hasMoreTokens()){
	                	String temp2 = st2.nextToken();
	                	RequiredItemIds.add(Long.parseLong(temp2.trim()));
	                }
                }
                items.add(new Item( Long.parseLong(id.trim()),
                                   new Point((int) (Double.valueOf(lat.trim()) * 1000000),(int) (Double.valueOf(lon.trim()) * 1000000)),
                                   name,
                                   description,
                                   (int) (Double.valueOf(pick_distance.trim()).longValue()),
                                   Integer.parseInt(point_value.trim()),
                                   iconURL,
                                   Long.parseLong(superId.trim()),
                                   RequiredItemIds
               					  )
               			);      
            }
        }catch(Exception e){
        }
		return items;
	}		

	public static ArrayList<Item> getSuperItem(Item itm){
		// server side code to get items in large circle around the person should go here
		// this list will only have items that can be seen (not the items that are in view)
		ArrayList<Item> items = new ArrayList<Item>();
		String temp;
		String line;
		String id = "0";
        String lon = "0";
        String lat = "0";
        String name = "0";
        String description = "0";
        String pick_distance = "0";
        String point_value="";
        String iconURL = "";
        String superId = "";
        String requiredItemsList ="";
        long itemId = itm.mId;
		
		try{
        	temp = "";
        	line = "";
        	String query = "http://khoaphung.com/android/getSuperItem.jsp?itemId="+itemId+"";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL (query).openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
            StringTokenizer st = new StringTokenizer(temp, ";");
            while(st.hasMoreTokens()){
                id = st.nextToken();
                lat = st.nextToken();
                lon = st.nextToken();
                name = st.nextToken();
                description = st.nextToken();
                pick_distance = st.nextToken();
                point_value = st.nextToken();
                iconURL = st.nextToken();
                superId = st.nextToken();
                if(superId.equals("none")){
                	superId = "0";
                }
                requiredItemsList = st.nextToken();
                ArrayList<Long> RequiredItemIds = new ArrayList<Long>();
                if(!requiredItemsList.equals("none")){
	                StringTokenizer st2 = new StringTokenizer(requiredItemsList, ",");
	                while(st2.hasMoreTokens()){
	                	String temp2 = st2.nextToken();
	                	RequiredItemIds.add(Long.parseLong(temp2.trim()));
	                }
                }
                items.add(new Item( Long.parseLong(id.trim()),
                                   new Point((int) (Double.valueOf(lat.trim()) * 1000000),(int) (Double.valueOf(lon.trim()) * 1000000)),
                                   name,
                                   description,
                                   (int) (Double.valueOf(pick_distance.trim()).longValue()),
                                   Integer.parseInt(point_value.trim()),
                                   iconURL,
                                   Long.parseLong(superId.trim()),
                                   RequiredItemIds
               					  )
               			);      
            }
        }catch(Exception e){
        }
		return items;
	}
	
	private void pickUpItem(Item i){
		String line="";
        String temp="";
        String onlineId = mGameData.getString("onlineId",null);
        String username = mGameData.getString("Username",null);
        long itemId = i.mId;
        
		try{
        	temp = "";
        	line = "";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL ("http://khoaphung.com/android/pickUpItem.jsp?onlineId="+onlineId+"&itemId="+itemId+"&username="+username+"").openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
		}catch(Exception e){
		}
	}
	
	public boolean checkFusedItem(Item i){
		String line="";
        String temp="";
        String onlineId = mGameData.getString("onlineId",null);
        String username = mGameData.getString("Username",null);
        long itemId = i.mId;
        
		try{
        	temp = "";
        	line = "";
            BufferedReader in = new BufferedReader (new InputStreamReader (new URL ("http://khoaphung.com/android/checkFusedItem.jsp?onlineId="+onlineId+"&itemId="+itemId+"&username="+username+"").openStream()));
            while((line = in.readLine()) != null ){
                temp = temp+line;
            }
		}catch(Exception e){
		}
		temp = temp.trim();
		if(temp.equals("true")){
			return true;
		}else{
			return false;
		}
	}
	
	private void generateOverlays(ArrayList<Item> visibleItems, ArrayList<Item> ofScreenItems){
		mLocationOverlay = new PersonOverlay(mCurrentLocation);
		mItemLocationsOverlay = new ItemsOverlay(visibleItems);
		mArrowDirectionsOverlay = new ArrowsOverlay(ofScreenItems);
	}
	private void activateOverlays(){
		for(Overlay o:mOverlayControl.getOverlays()){
			mOverlayControl.activate(o, false);
		}
	}

    /**
     * Overlay to draw person icon
     * @author Hari
     *
     */
    protected class PersonOverlay extends Overlay {
    	private Point mLoc;	
		public PersonOverlay(Point loc) {
			mLoc = new Point(loc.getLatitudeE6(),loc.getLongitudeE6());
		}
    	@Override
        public void draw(Canvas canvas, PixelCalculator calc, boolean shadow) {
             super.draw(canvas, calc, shadow);
             Paint p1 = new Paint();
             p1.setStyle(Style.STROKE);
             p1.setARGB(255, 0, 0, 255);
             p1.setAntiAlias(true);
             
             int[] xyCoords = new int[2];
             calc.getPointXY(mLoc, xyCoords);
             
             // draws the box icon at the location of the item
             Bitmap bit = BitmapFactory.decodeResource(getResources(), R.drawable.man);
             canvas.drawBitmap(bit, xyCoords[0]-bit.getWidth()/2, xyCoords[1]-bit.getHeight()/2, p1);
             canvas.drawCircle(xyCoords[0], xyCoords[1], 20, p1);
             //showAlert("perover",0,xyCoords[0]+","+xyCoords[1],"OK",true);
             // Animate to the center on your location
             //mMapControl.centerMapTo(mLoc,true);
        }
    }
    
    /**
     * Overlay to draw the items in view
     * @author Hari
     *
     */
    protected class ItemsOverlay extends Overlay {
    	private ArrayList<Item> mItems;	
		public ItemsOverlay(ArrayList<Item> items) {
			mItems = new ArrayList<Item>(items);
		}
    	@Override
        public void draw(Canvas canvas, PixelCalculator calc, boolean shadow) {
             super.draw(canvas, calc, shadow);
             Paint p1 = new Paint();
             p1.setStyle(Style.FILL);
             p1.setARGB(255, 0, 155, 0);
             p1.setAntiAlias(true);
             Paint p2 = new Paint();
             p2.setStyle(Style.STROKE);
             p2.setARGB(255, 0, 155, 0);
             p2.setAntiAlias(true);
             
             int[] xyCoords = new int[2];
             for (Item i : mItems){
            	 calc.getPointXY(i.mCoords,xyCoords);            	 
            	 canvas.drawCircle(xyCoords[0], xyCoords[1], 5, p1);
            	 canvas.drawCircle(xyCoords[0], xyCoords[1], i.mPickUpRadius, p2);
             } 
        }
    }
    /**
     * Overlay to draw the arrows towards items out of view
     * @author Hari
     *
     */
    protected class ArrowsOverlay extends Overlay {
    	private ArrayList<Item> mItems;	
		public ArrowsOverlay(ArrayList<Item> items) {
			mItems = new ArrayList<Item>(items);
		}
    	@Override
        public void draw(Canvas canvas, PixelCalculator calc, boolean shadow) {
             super.draw(canvas, calc, shadow);
             Paint p1 = new Paint(); // for the rectangle
             p1.setStyle(Style.STROKE);
             p1.setARGB(100, 255, 255, 255);
             p1.setStrokeWidth(25);
             p1.setAntiAlias(true);
             canvas.drawRect(mMap.getLeft(), mMap.getTop(),  
			 		 mMap.getRight(),mMap.getBottom(), p1);
	
             Paint p2 = new Paint();
             p2.setStyle(Style.FILL);
             p2.setARGB(255, 255, 0, 0);
             p2.setAntiAlias(true);
                      
             int[] xyCoords = new int[2];
             for (Item i : mItems){
            		 calc.getPointXY(i.mCoords,xyCoords);
            		 if(xyCoords[0] < mMap.getLeft()){
            			 xyCoords[0] = mMap.getLeft()+5;
            		 }else if(xyCoords[0] > mMap.getRight()){
            			 xyCoords[0] = mMap.getRight()-5;
            		 }
            		 
            		 if(xyCoords[1] < mMap.getTop()){
            			 xyCoords[1] = mMap.getTop()+5;
            		 }else if(xyCoords[1] > mMap.getBottom()){
            			 xyCoords[1] = mMap.getBottom()-5;
            		 }
            		 canvas.drawCircle(xyCoords[0], xyCoords[1], 5, p2);
             } 
        }
    }
 	public static class Item {
 		
 		// the delimiter for individual values
 		final static public String SPLIT_VALUES = "*@*";
 		// the delimiter for values in an array
 		final static public String SPLIT_REQS = "*;*";
 		
		public long mId;
		public Point mCoords;
		public String mName;
		public String mDescription;
		public int mPickUpRadius;
		public int mPointValue;
		public String mIconUrl;
		
		public long mSuperId;
		public ArrayList<Long> mRequiredItemIds;
		
		public Item(long id, Point coords, String name, String description, 
						int pickupdist, int points, String iconurl){
			this.mId = id;
			this.mCoords = new Point(coords.getLatitudeE6(),coords.getLongitudeE6());
			this.mName = new String(name);
			this.mDescription = new String(description);
			this.mPickUpRadius = pickupdist;
			this.mPointValue = points;
			
			this.mIconUrl = (iconurl != null)? new String(iconurl) : null;
			
			this.mSuperId = 0;
			this.mRequiredItemIds = null;
		}
		
		public Item(long id, Point coords, String name, String description, 
						int pickupdist, int points, String iconurl, 
						long superid, ArrayList<Long> req){
			this.mId = id;
			this.mCoords = new Point(coords.getLatitudeE6(),coords.getLongitudeE6());
			this.mName = new String(name);
			this.mDescription = new String(description);
			this.mPickUpRadius = pickupdist;
			this.mPointValue = points;
			
			this.mIconUrl = (iconurl != null)? new String(iconurl) : null;
			
			this.mSuperId = superid;
			this.mRequiredItemIds = new ArrayList<Long>(req);
		}
		
		
		public Item(String toStringItem){
			String[] pieces = toStringItem.split(SPLIT_VALUES);
			this.mId = Long.parseLong(pieces[0]);
			this.mCoords = new Point(Integer.parseInt(pieces[1],10),
										Integer.parseInt(pieces[2],10));
			this.mName = new String(pieces[3]);
			this.mDescription = new String(pieces[4]);
			//Toast.makeText(MapBrowser.this, pieces[5], Toast.LENGTH_SHORT).show();
			this.mPickUpRadius = Integer.parseInt(pieces[5],10);

			this.mIconUrl = (pieces[6] != "")? new String(pieces[6]) : null;
			
			this.mSuperId = Long.parseLong(pieces[7]);
			if(pieces[8] == ""){
				this.mRequiredItemIds = null;
			}else{
				String[] vals = pieces[8].split(SPLIT_REQS);
				for (String v: vals){
					if(v != ""){
						this.mRequiredItemIds.add(Long.parseLong(v));
					}
				}
			}
		}
		
		@Override
		public String toString() {
			String s = SPLIT_VALUES; // getting the split string
			String arrayVals = null;
			String t = SPLIT_REQS; // split the array by a delimiter
			if(mRequiredItemIds != null && mRequiredItemIds.size()>0){
				arrayVals = ((Long)(mRequiredItemIds.get(0))).toString();
				for (int i = 1; i<mRequiredItemIds.size(); i++){
					arrayVals += t+((Long)(mRequiredItemIds.get(i))).toString();
				}
			}
			
			return mId+s+mCoords.getLatitudeE6()+s+mCoords.getLongitudeE6()
			+s+mName+s+mDescription+s+mPickUpRadius+s+mIconUrl
			+s+mSuperId+s+arrayVals;
		}
	}    
}
