package edu.berkeley.cs160.group7.BeenThereDoneThatHiFi;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Gallery;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemSelectedListener;

public class NewAccount extends Activity {

	// the views available to account creation
	private static final int NEWACCOUNT_USERNAME_VIEW = 0;
	private static final int NEWACCOUNT_AVATAR_VIEW = 1;
	
	
	// the button and text field in the username entry view
	private Button mCreateUserButton;
	private EditText mUserText;
	
	// the grid in the avatar selection view
	private GridView mAvatarGrid;
	
    // misc info regarding the activity...mostly for lifecycle handling
	private int mCurrentView = NEWACCOUNT_USERNAME_VIEW;
	private String mCurrentUsername = null; // the name in the edittext field
	private int mCurrentAvatar = -1;
	private View mCurrentFocus = null;
	
	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		
		if(icicle != null){
			mCurrentView = icicle.getInt("current_view");
			mCurrentUsername = icicle.getString("Username");
			mCurrentAvatar = icicle.getInt("Avatar");

			changeView(mCurrentView);
			mCurrentFocus = findViewById(icicle.getInt("focus_id"));
			mCurrentFocus.requestFocus();
		}else{
			changeView(mCurrentView);
		}
		
	}
	
	@Override
	protected void onFreeze(Bundle outState) {
		super.onFreeze(outState);
		outState.putInt("current_view", mCurrentView);
		if(mCurrentView == NEWACCOUNT_USERNAME_VIEW){
			mCurrentUsername = mUserText.getText().toString();
		}else{
			mCurrentAvatar = mAvatarGrid.getSelectedItemPosition();
		}
		mCurrentFocus = getCurrentFocus();
		outState.putInt("Avatar", mCurrentAvatar);
		outState.putString("Username",mCurrentUsername);
		outState.putInt("focus_id", mCurrentFocus.getId());
	}

	@Override
	protected void onPause() {
		super.onPause();
		mCurrentFocus = getCurrentFocus();
		if(mCurrentView == NEWACCOUNT_USERNAME_VIEW){
			mCurrentUsername = mUserText.getText().toString();
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		if(mCurrentFocus != null){
			mCurrentFocus.requestFocus();
		}
		if(mCurrentView == NEWACCOUNT_USERNAME_VIEW){
			mUserText.setText(mCurrentUsername);
		}
	}

	private void changeView(int viewid){
		if(viewid == NEWACCOUNT_USERNAME_VIEW){
			setContentView(R.layout.newaccount_username);
			mCurrentView = NEWACCOUNT_USERNAME_VIEW;
			mUserText = (EditText) findViewById(R.id.enter_user_textfield);
			
			mCreateUserButton = (Button) findViewById(R.id.create_user_button);
			mCreateUserButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							mCurrentUsername = mUserText.getText().toString();
							Toast t;
							if(mCurrentUsername.length() == 0){
								t = Toast.makeText(v.getContext(),"Enter a Username.",Toast.LENGTH_SHORT);
								t.setGravity(Gravity.BOTTOM);
								t.show();
							}else if(mCurrentUsername.contains(" ")){
								t = Toast.makeText(v.getContext(),"No Spaces allowed.",Toast.LENGTH_SHORT);
								t.setGravity(Gravity.BOTTOM);
								t.show();
							}else{
								changeView(NEWACCOUNT_AVATAR_VIEW);
							}
						}
					});
			
			if(mCurrentUsername == null || mCurrentUsername.length() == 0){
				mUserText.setHint("Enter Username");
				mUserText.requestFocus();
			}else{
				mUserText.setText(mCurrentUsername);
				mCreateUserButton.requestFocus();
			}
			mCurrentFocus = getCurrentFocus();
		}
		if(viewid == NEWACCOUNT_AVATAR_VIEW){
			setContentView(R.layout.newaccount_avatar);
			mCurrentView = NEWACCOUNT_AVATAR_VIEW;
			
			mAvatarGrid = (GridView) findViewById(R.id.avatar_grid);
			mAvatarGrid.setAdapter(new ImageAdapter(this));
			
			
			mAvatarGrid.setOnItemSelectedListener(new OnItemSelectedListener(){
				@SuppressWarnings("unchecked")
				public void onItemSelected(AdapterView parent, View v, int position, long id){
					mCurrentAvatar = position;
				}
				@SuppressWarnings("unchecked")
				public void onNothingSelected(AdapterView parent){
					mCurrentAvatar = -1;
				}
			});
			mAvatarGrid.setOnItemClickListener(new OnItemClickListener() {
				@SuppressWarnings("unchecked")
	            public void onItemClick(AdapterView parent, View v, int position, long id) {
	            	mCurrentAvatar = position;
	            	Bundle res = new Bundle();
	            	//Create Unique identifier for online ranking
					String onlineId = String.valueOf(Math.round(Math.random()*99999999)) + System.currentTimeMillis();
				
					res.putString("onlineId", onlineId);
	            	res.putString("Username",mCurrentUsername);
	            	res.putInt("Avatar",((Integer)((ImageAdapter)parent.getAdapter()).getItem(mCurrentAvatar)).intValue());
	            	setResult(RESULT_OK,"onlineId, Username, Avatar, ItemsCount, PointsCount, Level",res);
	            	
	            	// resetting variables so the wizard runs properly if game is reset
	            	mCurrentView = NEWACCOUNT_USERNAME_VIEW;
	            	mCurrentUsername = null;
	            	mUserText.setText(mCurrentUsername); // purging the text box in onPause will try to save it
	            	mCurrentAvatar = -1;
	            	mCurrentFocus = null;
	            	
	            	finish();
	            }
	        });
			mAvatarGrid.requestFocus();
			if(mCurrentAvatar != -1){
				mAvatarGrid.setSelection(mCurrentAvatar);
			}
			mCurrentFocus = getCurrentFocus();
			
		}
		
	}
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK:
			if(mCurrentView == NEWACCOUNT_AVATAR_VIEW){
				changeView(NEWACCOUNT_USERNAME_VIEW);
			}else{
				finish();
			}
			return true;
		default:
			return super.onKeyDown(keyCode, event);
		}
	}
	

	public class ImageAdapter extends BaseAdapter {
        private Integer[] mImages = {
        		R.drawable.none, R.drawable.icon,
        		R.drawable.bee, R.drawable.fish,
        		R.drawable.flower, R.drawable.pawprint,
        		R.drawable.peacesign, R.drawable.rabbit,
        		R.drawable.smiley, R.drawable.snake,
        		R.drawable.yinyang, R.drawable.basketball,
        		R.drawable.cal
        		
        };
        private Context mContext;
        public ImageAdapter(Context c) {
            mContext = c;
        }
        
        //@Override
        public int getCount() {
            return mImages.length;
        }
        
        //@Override
        public Object getItem(int position) {
            return mImages[position];
        }
        
        public Context getContext() {
        	return mContext;
        }
        
        //@Override
        public long getItemId(int position) {
            return position;
        }
        
        //@Override
        public View getView(int position, View convertView, ViewGroup parent) {
        	ImageView i = new ImageView(mContext);
            i.setImageResource(mImages[position]);
            
            //Borrowed from ApiDemo/View/Grid2.java
            i.setLayoutParams(new Gallery.LayoutParams(45, 45));
            i.setAdjustViewBounds(false);
            i.setScaleType(ImageView.ScaleType.CENTER_CROP);
            i.setPadding(1, 1, 1, 1);
            return i;
        }
    }
}
