package edu.berkeley.cs160.group7.BeenThereDoneThatHiFi;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.AudioSystem;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.Gallery;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.Toast;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemSelectedListener;

public class SettingsMenu extends Activity {
	
	// Shared preferences file where all the game data like username will be stored
	public static SharedPreferences mGameData = null;
    public static SharedPreferences.Editor mGameDataEditor = null;
	
    public static final String PREF_SOUND = "Settings.SoundOccurs";
    public static final String PREF_VOLUME = "Settings.Volume";
    public static final String PREF_ALERTMODE = "Settings.AlertMode";
    public static final String PREF_ALERTSOUND = "Settings.AlertSound";
    public static final String PREF_ALERTLEVEL = "Settings.AlertLevel";
    public static final String PREF_ALERTSTATUS = "Settings.AlertsStatus";
    
	
	private static final int SYSTEM_SOUND_VIEW = 0;
	public static final int ITEM_ALERT_OFF_VIEW = 1;
	public static final int ITEM_ALERT_ON_VIEW = 2;
	private static final int CHANGE_AVATAR_VIEW = 3;
	private static final int SETTINGS_MENU_VIEW = 4;
	
	private static final String[] mAlertMode = {"Task Bar", "Launch Application"};
	private static final String[] mAlertSound = {"Never", "With Profile","Always"};
	private static final String[] mAlertLevel = {"Close", "Far"};
	private static final String[] mSoundOccurs = {"Never", "Always"};
	public static final String[] mVolume = {"Vibrate", "Soft", "Medium", "Loud"};
	
	private Button mSystemSoundButton, mItemAlertButton, mChangeAvatarButton, mResetButton;
	private Button mSaveSettingsButton, mMainMenuButton, mToggleAlertsButton;
	private Spinner sAlertMode, sAlertSound, sAlertLevel, sSoundOccurs, sVolume;

	private int mCurrentAvatar = -1;
	private GridView mAvatarGrid;
	
	private int mCurrentView;
	private View mCurrentFocus = null;

	private ArrayAdapter<String> adapter1, adapter2, adapter3;

	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		mGameData = getSharedPreferences(Launcher.STORAGE_FILE,MODE_PRIVATE);
        mGameDataEditor = mGameData.edit();

		if(icicle != null){
			mCurrentView = icicle.getInt("current_view");
			changeView(mCurrentView);
			mCurrentFocus = findViewById(icicle.getInt("focus_id"));
			mCurrentFocus.requestFocus();
		}else{
			changeView(SETTINGS_MENU_VIEW);
		}  
	}
	
	@Override
	protected void onFreeze(Bundle outState) {
		super.onFreeze(outState);
		outState.putInt("current_view", mCurrentView);
		mCurrentFocus = getCurrentFocus();
		if (mCurrentFocus != null) {
			outState.putInt("focus_id", mCurrentFocus.getId());
		}
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		mCurrentFocus = getCurrentFocus();
	}

	@Override
	protected void onResume() {
		super.onResume();
		if(mCurrentFocus != null){
			mCurrentFocus.requestFocus();
		}
	}

	//Button Execution
	private void changeView(int viewid){
		//***********************************************************
		if(viewid == SYSTEM_SOUND_VIEW){
			mCurrentView = SYSTEM_SOUND_VIEW;
			setContentView(R.layout.settings_menu_game_sound);
			mMainMenuButton = (Button) findViewById(R.id.main_menu_button);
			mSaveSettingsButton = (Button) findViewById(R.id.save_settings_button);
			
			sSoundOccurs = (Spinner) findViewById(R.id.sound_occurs_dropdown);
			sSoundOccurs.requestFocus();
			
			adapter1 = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mSoundOccurs);
			sSoundOccurs.setAdapter(adapter1);
			sSoundOccurs.setSelection(mGameData.getInt(PREF_SOUND,0));			
			
			sVolume = (Spinner) findViewById(R.id.volume_dropdown);	
			adapter2 = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mVolume);
			sVolume.setAdapter(adapter2);
			sVolume.setSelection(mGameData.getInt(PREF_VOLUME,0));					
	
			mMainMenuButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							finish();
						}
					});
			mSaveSettingsButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							mGameDataEditor.putInt(PREF_SOUND, sSoundOccurs.getSelectedItemPosition());
							mGameDataEditor.putInt(PREF_VOLUME, sVolume.getSelectedItemPosition());
							mGameDataEditor.commit();
							if( String.valueOf(sSoundOccurs.getSelectedItem()).equals(mVolume[0])){
								AudioSystem.setVolume(AudioSystem.STREAM_SYSTEM, 0);			
							}
							if( String.valueOf(sSoundOccurs.getSelectedItem()).equals(mVolume[1])){
								AudioSystem.setVolume(AudioSystem.STREAM_SYSTEM, 33);			
							}
							if( String.valueOf(sSoundOccurs.getSelectedItem()).equals(mVolume[2])){
								AudioSystem.setVolume(AudioSystem.STREAM_SYSTEM, 66);			
							}
							if( String.valueOf(sSoundOccurs.getSelectedItem()).equals(mVolume[3])){
								AudioSystem.setVolume(AudioSystem.STREAM_SYSTEM, 100);			
							}
							
							Toast.makeText(SettingsMenu.this,"Game sound SAVED.",Toast.LENGTH_SHORT).show();
						
						}
					});
			
		}
		//***********************************************************
		if(viewid == ITEM_ALERT_ON_VIEW){
			mCurrentView = ITEM_ALERT_ON_VIEW;
			setContentView(R.layout.settings_menu_item_alert);
			
			mMainMenuButton = (Button) findViewById(R.id.main_menu_button);
			mSaveSettingsButton = (Button) findViewById(R.id.save_settings_button);
			mToggleAlertsButton = (Button) findViewById(R.id.alerts_off_button);
			
			sAlertMode = (Spinner) findViewById(R.id.alert_mode_dropdown);
			sAlertMode.requestFocus();
			
			adapter1 = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAlertMode);
			sAlertMode.setAdapter(adapter1);
			sAlertMode.setSelection(mGameData.getInt(PREF_ALERTMODE,0));

			sAlertSound = (Spinner) findViewById(R.id.alert_sound_dropdown);
			adapter2 = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAlertSound);
			sAlertSound.setAdapter(adapter2);
			sAlertSound.setSelection(mGameData.getInt(PREF_ALERTSOUND,0));
			
			sAlertLevel = (Spinner) findViewById(R.id.alert_level_dropdown);	
			adapter3 = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAlertLevel);
			sAlertLevel.setAdapter(adapter3);
			sAlertLevel.setSelection(mGameData.getInt(PREF_ALERTLEVEL,0));

			mMainMenuButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						finish();
					}
				});
			mSaveSettingsButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							mGameDataEditor.putInt(PREF_ALERTSOUND, sAlertSound.getSelectedItemPosition());
							mGameDataEditor.putInt(PREF_ALERTLEVEL, sAlertLevel.getSelectedItemPosition());
							mGameDataEditor.putInt(PREF_ALERTMODE, sAlertMode.getSelectedItemPosition());
							mGameDataEditor.commit();
							Toast.makeText(SettingsMenu.this,"Item Alert SAVED.",Toast.LENGTH_SHORT).show();
						
						}
					});
			mToggleAlertsButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							mGameDataEditor.putInt(PREF_ALERTSTATUS, ITEM_ALERT_OFF_VIEW);
							mGameDataEditor.commit();
				            stopService(new Intent(SettingsMenu.this, 
				            						ItemScanner.class));
							changeView(ITEM_ALERT_OFF_VIEW);
						}
					});
			
		}
		//***********************************************************
		if(viewid == ITEM_ALERT_OFF_VIEW){
			mCurrentView = ITEM_ALERT_OFF_VIEW;
			setContentView(R.layout.settings_menu_item_alerts_off);
			
			mMainMenuButton = (Button) findViewById(R.id.main_menu_button);
			mToggleAlertsButton = (Button) findViewById(R.id.alerts_on_button);
			mToggleAlertsButton.requestFocus();
			mMainMenuButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						finish();
					}
				});
			mToggleAlertsButton.setOnClickListener(
					new OnClickListener() {
						public void onClick(View v) {
							mGameDataEditor.putInt(PREF_ALERTSTATUS, ITEM_ALERT_ON_VIEW);
							mGameDataEditor.commit();
				            startService(new Intent(SettingsMenu.this, 
				            						ItemScanner.class), null);
							changeView(ITEM_ALERT_ON_VIEW);
						}
					});
			
		}
		//***********************************************************
		if(viewid == CHANGE_AVATAR_VIEW){
			setContentView(R.layout.newaccount_avatar);
			mCurrentView = CHANGE_AVATAR_VIEW;
			
			mAvatarGrid = (GridView) findViewById(R.id.avatar_grid);
			mAvatarGrid.setAdapter(new ImageAdapter(this));
			
			
			mAvatarGrid.setOnItemSelectedListener(new OnItemSelectedListener(){
				@SuppressWarnings("unchecked")
				public void onItemSelected(AdapterView parent, View v, int position, long id){
					mCurrentAvatar = position;
				}
				@SuppressWarnings("unchecked")
				public void onNothingSelected(AdapterView parent){
					mCurrentAvatar = -1;
				}
			});
			mAvatarGrid.setOnItemClickListener(new OnItemClickListener() {
				@SuppressWarnings("unchecked")
	            public void onItemClick(AdapterView parent, View v, int position, long id) {
	            	mCurrentAvatar = position;
	            	mGameDataEditor.putInt("Avatar",  ((Integer)((ImageAdapter)parent.getAdapter()).getItem(mCurrentAvatar)).intValue()   );
	            	mGameDataEditor.commit();
	            	Toast.makeText(SettingsMenu.this,"Icon CHANGED.",Toast.LENGTH_SHORT).show();
	            	mCurrentAvatar = -1;
	          
	            	changeView(SETTINGS_MENU_VIEW );
	            }
	        });
			mAvatarGrid.requestFocus();
			if(mCurrentAvatar != -1){
				mAvatarGrid.setSelection(mCurrentAvatar);
			}
	//		mCurrentFocus = getCurrentFocus();
		}

		//***********************************************************
		if(viewid == SETTINGS_MENU_VIEW ){
			mCurrentView = SETTINGS_MENU_VIEW ;
			setContentView(R.layout.settings_menu);
			mSystemSoundButton = (Button) findViewById(R.id.system_sound_button);
			mItemAlertButton = (Button) findViewById(R.id.item_alert_button);
			mChangeAvatarButton = (Button) findViewById(R.id.change_avatar_button);
			mResetButton = (Button) findViewById(R.id.reset_button);
			mMainMenuButton = (Button) findViewById(R.id.main_menu_button);
			mSystemSoundButton.requestFocus();
		
			//Button Actions
			mMainMenuButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						finish();
					}
				});
			mSystemSoundButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						changeView(SYSTEM_SOUND_VIEW);
					}
				});
			mItemAlertButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						//Toast.makeText(SettingsMenu.this, mGameData.getInt(PREF_ALERTSTATUS,"ERR"), Toast.LENGTH_SHORT).show();
						if(mGameData.getInt(PREF_ALERTSTATUS,ITEM_ALERT_OFF_VIEW) == ITEM_ALERT_OFF_VIEW){	
							changeView(ITEM_ALERT_OFF_VIEW);
						}else{
							changeView(ITEM_ALERT_ON_VIEW);
						}
					}
				});
			mChangeAvatarButton.setOnClickListener(
				new OnClickListener() {
					public void onClick(View v) {
						changeView(CHANGE_AVATAR_VIEW);
					}
				});
			mResetButton.setOnClickListener(
	                new OnClickListener() {
	                    public void onClick(View v) {
	                        reset_game_prompt();
	                    }
	                });

		}
		
	}
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK:
			if(mCurrentView == ITEM_ALERT_ON_VIEW || mCurrentView == ITEM_ALERT_OFF_VIEW 
					|| mCurrentView == SYSTEM_SOUND_VIEW || mCurrentView == CHANGE_AVATAR_VIEW ){
				changeView(SETTINGS_MENU_VIEW);
			}else{
				finish();
			}
			return true;
		default:
			return super.onKeyDown(keyCode, event);
		}
	}
	
	private void reset_game_prompt(){	
    	DialogInterface.OnClickListener lis = new DialogInterface.OnClickListener(){
    		public void onClick(DialogInterface dialog, int which){
    			if(which == -1){
    				setResult(RESULT_FIRST_USER);
    				finish();
    			}else{
    				Toast.makeText(SettingsMenu.this,"Reset has been aborted.",Toast.LENGTH_SHORT).show();
    			}
    		}
    	};	
    	showAlert("Reset Game?",0,"Warning: Pressing \"Yes\" will reset all current user data and is not recoverable.",
    			  "Yes",lis, "No",lis,false,null);
    }
	
	public class ImageAdapter extends BaseAdapter {
        private Integer[] mImages = {
        		R.drawable.none, R.drawable.icon,
        		R.drawable.bee, R.drawable.fish,
        		R.drawable.flower, R.drawable.pawprint,
        		R.drawable.peacesign, R.drawable.rabbit,
        		R.drawable.smiley, R.drawable.snake,
        		R.drawable.yinyang, R.drawable.basketball,
        		R.drawable.cal
        		
        };
        private Context mContext;
        public ImageAdapter(Context c) {
            mContext = c;
        }
        
        // count of how many images were parsed
        //@Override
        public int getCount() {
            return mImages.length;
        }
        
        // accessor for the images (Drawables)
        //@Override
        public Object getItem(int position) {
            return mImages[position];
        }
        
        // accessor for the context in which it is drawing
        public Context getContext() {
        	return mContext;
        }
        
        // gets the position of the selected
        //@Override
        public long getItemId(int position) {
            return position;
        }
        
        // creates a gallery view for the gridview
        //@Override
        public View getView(int position, View convertView, ViewGroup parent) {
        	ImageView i = new ImageView(mContext);
            i.setImageResource(mImages[position]);
            
            //Borrowed from ApiDemo/View/Grid2.java
            i.setLayoutParams(new Gallery.LayoutParams(45, 45));
            i.setAdjustViewBounds(false);
            i.setScaleType(ImageView.ScaleType.CENTER_CROP);
            i.setPadding(1, 1, 1, 1);
            return i;
        }
    }
}
